SystemOrganization addCategory: #NBSQLite3!
SystemOrganization addCategory: 'NBSQLite3-Core'!
SystemOrganization addCategory: 'NBSQLite3-Core-Callbacks'!
SystemOrganization addCategory: 'NBSQLite3-Core-Exceptions'!
SystemOrganization addCategory: 'NBSQLite3-Examples'!
SystemOrganization addCategory: 'NBSQLite3-Glorp'!
SystemOrganization addCategory: 'NBSQLite3-Tests'!
SystemOrganization addCategory: 'NBSQLite3-Tests-Glorp'!

TestCase subclass: #NBSQLite3BaseConnectionTest
	instanceVariableNames: 'db db2 res res2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Tests'!

!NBSQLite3BaseConnectionTest commentStamp: 'PierceNg 3/29/2014 20:18' prior: 0!
I provide the test cases for NBSQLite3BaseConnection.!

!NBSQLite3BaseConnectionTest methodsFor: 'running' stamp: 'PierceNg 3/30/2014 16:21'!
setUp 

	super setUp.
	db := NBSQLite3BaseConnection on: ':memory:'.
	db open.
! !

!NBSQLite3BaseConnectionTest methodsFor: 'running' stamp: 'TorstenBergmann 11/2/2014 20:50'!
tearDown
	
	[ db close ] on: NBSQLite3Misuse do: [ ].
	super tearDown.
! !

!NBSQLite3BaseConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/5/2014 13:47'!
testBadOpen

	db2 := NBSQLite3BaseConnection on: '/nosuchfile'.
	self should: [ db2 open ] raise: NBSQLite3Error	.
	self assert: (db2 isOpen = false)! !

!NBSQLite3BaseConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/5/2014 19:43'!
testBasicExecuteOk

	res := db basicExecute: 'create table x (xk integer, xv integer);'.
	self assert: (res = 0).
	res := db basicExecute: 'drop table x;'.
	self assert: (res = 0).
	! !

!NBSQLite3BaseConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/5/2014 19:43'!
testBasicExecuteSyntaxError

	self should: [ db basicExecute: 'create table,;' ]
		raise: NBSQLite3Error
	! !

!NBSQLite3BaseConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/5/2014 19:38'!
testColumnNames
	| columns |
	
	res := db basicExecute: 'create table x (xk integer, xv integer);'.
	self assert: (res = 0).
	columns := db columnNamesFor: 'x'.
	self assert: (columns size = 2).
	self assert: (columns first = 'xk').
	self assert: (columns second = 'xv').
	
! !

!NBSQLite3BaseConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/5/2014 19:39'!
testColumnNamesBobbyGo
	"In this test, Bobby Tables strikes naive SQL string construction."
	| tables bobby |
	
	res := db basicExecute: 'create table x (xk integer, xv integer);'.
	self assert: (res = 0).
	res := db basicExecute: 'create table students (sk integer, sname varchar);'.
	self assert: (res = 0).

	tables := db tableNames.
	self assert: (tables size = 2).
	self assert: (tables first = 'x').	
	self assert: (tables second = 'students').	

	bobby := 'x); drop table students; --'.	
	res := db basicExecute: 'pragma table_info(', bobby, ')'.
	self assert: (res = 0).

	tables := db tableNames.
	self assert: (tables size = 1).
	self assert: (tables first = 'x').	

! !

!NBSQLite3BaseConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/5/2014 19:39'!
testColumnNamesBobbyStop
	"In this test, Bobby Tables strikes again, but fails, because #columnNamesFor: uses parameter binding."
	| columns tables |
	
	res := db basicExecute: 'create table x (xk integer, xv integer);'.
	self assert: (res = 0).
	res := db basicExecute: 'create table students (sk integer, sname varchar);'.
	self assert: (res = 0).

	tables := db tableNames.
	self assert: (tables size = 2).
	self assert: (tables first = 'x').	
	self assert: (tables second = 'students').	
	
	columns := db columnNamesFor: 'x; drop table students; --'.
	self assert: (columns isNil).

	tables := db tableNames.
	self assert: (tables size = 2).
	self assert: (tables first = 'x').	
	self assert: (tables second = 'students').	

! !

!NBSQLite3BaseConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/5/2014 19:39'!
testColumnNamesNoSuchTable
	| columns |
	
	res := db basicExecute: 'create table x (xk integer, xv integer);'.
	self assert: (res = 0).
	columns := db columnNamesFor: 'y'.
	self assert: (columns isNil).! !

!NBSQLite3BaseConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/12/2014 23:36'!
testDataValuesAvailable
	| s | 
	
	res := db basicExecute: 'create table x (xk integer primary key, iv integer, tv text);'.
	self assert: (res = 0).

	res := db basicExecute: 'insert into x values (NULL, 1, "duh");'.	
	self assert: (res = 0).
	
	s := db prepare: 'select * from x'.
	s basicExecute: [  :row |
		self assert: (row dataValuesAvailable = 3) ].
	s finalize.
	! !

!NBSQLite3BaseConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/5/2014 14:07'!
testDoubleClose
	
	db close.
	[ db close ] on: NBSQLite3Misuse do: [ ]
	"#tearDown will close the db again, and is similarly guarded."! !

!NBSQLite3BaseConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/12/2014 23:36'!
testFloat
	| s |
	
	res := db basicExecute: 'create table x (xk integer primary key, xv real);'.
	self assert: (res = 0).
	res := db basicExecute: 'insert into x values (NULL, 1.141);'.
	self assert: (res = 0).
	
	s := db prepare: 'select * from x'.
	s basicExecute: [ :row |
		self assert: ((row nameOfColumn: 1) = 'xv').
		self assert: ((row floatAt: 1) = 1.141) ].
	s finalize.
! !

!NBSQLite3BaseConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/12/2014 23:36'!
testObjectBlob
	| data s | 
	
	data := OrderedCollection with: 1 with: 'two' with: 3.3.
	
	res := db basicExecute: 'create table x (xk integer primary key, xv blob);'.
	self assert: (res = 0).

	res := db basicExecute: 'begin'.
	self assert: (res = 0).
	s := db prepare: 'insert into x values (NULL, ?)'.
	s at: 1 putObject: data.
	s step. s finalize.
	res := db basicExecute: 'commit'.
	self assert: (res = 0).
	
	s := db prepare: 'select xv from x'.
	s basicExecute: [  :row |
		| v |
		v := row objectAt: 0.
		self assert: ((v size) = 3).
		self assert: ((v first) = 1).
		self assert: ((v second) = 'two').
		self assert: ((v third) = 3.3) ].
	s finalize.
	! !

!NBSQLite3BaseConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/12/2014 23:37'!
testPrepareStep
	| s i | 
	
	res := db basicExecute: 'create table x (xk integer primary key, iv integer, tv text);'.
	self assert: (res = 0).

	res := db basicExecute: 'begin'.
	self assert: (res = 0).
	s := db prepare: 'insert into x values (NULL, ?, ?)'.
	1 to: 10 do: [ :x |
		s at: 1 putInteger: x * x.
		s at: 2 putString: x asString, ' * ', x asString.
		s step. s clearBindings. s reset. ].		
	s finalize.
	res := db basicExecute: 'commit'.
	self assert: (res = 0).
	
	s := db prepare: 'select * from x'.
	s basicExecute: [  :row |
		i := row integerAt: 0.
		self assert: ((row integerAt: 1) = (i * i)).
		self assert: ((row stringAt: 2) = (i asString, ' * ', i asString)) ].
	s finalize.
	! !

!NBSQLite3BaseConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/12/2014 23:37'!
testResultSetColumnIntrospection
	| s | 

	res := db basicExecute: 'create table x (xk integer primary key, iv integer, tv text);'.
	self assert: (res = 0).

	res := db basicExecute: 'insert into x values (NULL, 1, "duh");'.	
	self assert: (res = 0).
	
	s := db prepare: 'select * from x'.
	s basicExecute: [  :row |
		self assert: ((row nameOfColumn: 0) = 'xk').
		self assert: ((row valueOfColumn: 0) = 1).
		self assert: ((row nameOfColumn: 1) = 'iv').
		self assert: ((row valueOfColumn: 1) = 1).
		self assert: ((row nameOfColumn: 2) = 'tv').
		self assert: ((row valueOfColumn: 2) = 'duh') ].
	s finalize.
	! !

!NBSQLite3BaseConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/5/2014 19:42'!
testTableNames
	| tables |
	
	res := db basicExecute: 'create table x (xk integer, xv integer);'.
	self assert: (res = 0).
	tables := db tableNames.
	self assert: (tables size = 1).
	self assert: (tables first = 'x')
	
! !

!NBSQLite3BaseConnectionTest methodsFor: 'tests' stamp: 'PierceNg 8/2/2014 10:02'!
testTracing
	| sql callback expected actual  |
	
	sql := #('create table if not exists x (xk integer primary key, iv integer, tv text)'
			'insert into x values (NULL, 1, ''two'')'
			'drop table x').
			
	expected := String streamContents: [ :s |
		s nextPutAll: sql first; cr;
			nextPutAll: sql second; cr;
			nextPutAll: sql third; cr ].
	
	actual := WriteStream on: String new.
	
	callback := NBSQLite3TraceCallback on: [ :appdata :sqltext |
		"We ignore appdata in this test, because we're passing a null pointer in."
		actual nextPutAll: sqltext; cr ].
	
	"The null pointer appears as appdata in the callback."
	db traceUsing: callback with: NBExternalAddress null. 
	
	res := db basicExecute: sql first.
	self assert: (res = 0).
	
	res := db basicExecute: sql second.
	self assert: (res = 0).

	res := db basicExecute: sql third.
	self assert: (res = 0).

	self assert: (actual contents = expected)
	
! !

TestCase subclass: #NBSQLite3ConnectionTest
	instanceVariableNames: 'db res rs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Tests'!

!NBSQLite3ConnectionTest methodsFor: 'running' stamp: 'PierceNg 4/13/2014 11:31'!
setUp 

	super setUp.
	db := NBSQLite3Connection on: ':memory:'.
	db open.
! !

!NBSQLite3ConnectionTest methodsFor: 'running' stamp: 'TorstenBergmann 11/2/2014 20:50'!
tearDown
	
	[ db close ] on: NBSQLite3Misuse do: [ ].
	super tearDown.! !

!NBSQLite3ConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/13/2014 12:36'!
testExecuteDelete
	
	res := db basicExecute: 'create table if not exists x (xk integer primary key, iv integer, tv text);'.
	self assert: (res = 0).
	
	rs := db execute: 'delete from x;'.
	self assert: (rs next isNil).
	rs close.
! !

!NBSQLite3ConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/13/2014 12:36'!
testExecuteDrop
	
	res := db basicExecute: 'create table if not exists x (xk integer primary key, iv integer, tv text);'.
	self assert: (res = 0).
	
	rs := db execute: 'drop table x;'.
	self assert: (rs next isNil).
	rs close.
! !

!NBSQLite3ConnectionTest methodsFor: 'tests' stamp: 'PierceNg 9/21/2014 12:44'!
testExecuteNoResult
	
	rs := db execute: 'create table if not exists x (xk integer primary key, iv integer, tv text);'.
	self assert: (rs next isNil).
	rs close.
	
	rs := db execute: 'delete from x;'.
	self assert: (rs next isNil).
	rs close.
	
	rs := db execute: 'drop table x;'.
	self assert: (rs next isNil).
	rs close.

! !

!NBSQLite3ConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/13/2014 11:00'!
testExecuteWithInsert
	
	res := db basicExecute: 'create table if not exists x (xk integer primary key, iv integer, tv text);'.
	self assert: (res = 0).
	
	res := db basicExecute: 'delete from x;'.
	self assert: (res = 0).	

	res := db beginTransaction.
	self assert: (res = 0).

	rs := db execute: 'insert into x values (NULL, ?, ?)' with: #(1 'two').
	self assert: (rs next isNil).
	rs close.

	res := db commitTransaction.
	self assert: (res = 0).
	! !

!NBSQLite3ConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/13/2014 11:00'!
testExecuteWithInsert2
	
	res := db basicExecute: 'create table if not exists x (xk integer primary key, iv integer, tv text);'.
	self assert: (res = 0).
	
	res := db basicExecute: 'delete from x;'.
	self assert: (res = 0).	

	res := db beginTransaction.
	self assert: (res = 0).

	rs := db execute: 'insert into x values (NULL, :first, :two)'
		with: (Dictionary newFromPairs: #(':first' 1 ':two' 2 ':nosuch' 3)).
	self assert: (rs next isNil).
	rs close.

	res := db commitTransaction.
	self assert: (res = 0).
	! !

!NBSQLite3ConnectionTest methodsFor: 'tests' stamp: 'PierceNg 7/31/2014 20:44'!
testExecuteWithInsertChanges

	res := db basicExecute: 'create table if not exists x (xk integer primary key, iv integer, tv text);'.
	self assert: (res = 0).
	
	res := db basicExecute: 'delete from x;'.
	self assert: (res = 0).	

	res := db beginTransaction.
	self assert: (res = 0).
	rs := db execute: 'insert into x values (NULL, ?, ?)' with: #(1 'two').
	self assert: (rs next isNil).
	rs close.
	res := db commitTransaction.
	self assert: (res = 0).
	
	self assert: (db changes = 1).
	self assert: (db totalChanges = 1).

	res := db beginTransaction.
	self assert: (res = 0).
	rs := db execute: 'insert into x values (NULL, ?, ?)' with: #(2 'too').
	self assert: (rs next isNil).
	rs close.
	res := db commitTransaction.
	self assert: (res = 0).
	
	self assert: (db changes = 1).
	self assert: (db totalChanges = 2).
! !

!NBSQLite3ConnectionTest methodsFor: 'tests' stamp: 'PierceNg 9/21/2014 12:49'!
testExecuteWithInsertNoDataThenSelect
	| row |
	
	res := db basicExecute: 'create table if not exists x (xk integer primary key);'.
	self assert: (res = 0).
	
	res := db basicExecute: 'delete from x;'.
	self assert: (res = 0).	

	res := db beginTransaction.
	self assert: (res = 0).

	rs := db execute: 'insert into x values (NULL)' with: #().
	self assert: (rs next isNil).
	rs close.

	res := db commitTransaction.
	self assert: (res = 0).

	rs := db execute: 'select * from x'.	
	row := rs next.
	self assert: ((row at: 'xk') = 1).
	rs close.

	 ! !

!NBSQLite3ConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/13/2014 12:45'!
testExecuteWithInsertThenSelect
	| row |
	
	res := db basicExecute: 'create table if not exists x (xk integer primary key, iv integer, tv text);'.
	self assert: (res = 0).
	
	res := db basicExecute: 'delete from x;'.
	self assert: (res = 0).	

	res := db beginTransaction.
	self assert: (res = 0).

	rs := db execute: 'insert into x values (NULL, ?, ?)' with: #(1 'two').
	self assert: (rs next isNil).
	rs close.

	res := db commitTransaction.
	self assert: (res = 0).

	rs := db execute: 'select * from x'.	
	row := rs next.
	self assert: ((row at: 'xk') = 1).
	self assert: ((row at: 'iv') = 1).
	self assert: ((row at: 'tv') = 'two').
	self assert: (rs next isNil).
	rs close.

	 ! !

!NBSQLite3ConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/13/2014 12:45'!
testExecuteWithInsertThenSelect2
	| row |
	
	res := db basicExecute: 'create table if not exists x (xk integer primary key, iv integer, tv text);'.
	self assert: (res = 0).
	
	res := db basicExecute: 'delete from x;'.
	self assert: (res = 0).	

	res := db beginTransaction.
	self assert: (res = 0).

	rs := db execute: 'insert into x values (NULL, ?, ?)' with: #(1 'two').
	self assert: (rs next isNil).
	rs close.

	rs := db execute: 'insert into x values (NULL, ?, ?)' with: #(2 'three').
	self assert: (rs next isNil).
	rs close.

	res := db commitTransaction.
	self assert: (res = 0).

	rs := db execute: 'select * from x'.	
	row := rs next.
	self assert: ((row at: 'xk') = 1).
	self assert: ((row at: 'iv') = 1).
	self assert: ((row at: 'tv') = 'two').
	row := rs next.
	self assert: ((row at: 'xk') = 2).
	self assert: ((row at: 'iv') = 2).
	self assert: ((row at: 'tv') = 'three').
	self assert: (rs next isNil).
	rs close.

	 ! !

!NBSQLite3ConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/12/2014 09:44'!
testExecuteWithToFail
	
	res := db basicExecute: 'create table if not exists x (xk integer primary key, iv integer, tv text);'.
	self assert: (res = 0).
	
	res := db basicExecute: 'delete from x;'.
	self assert: (res = 0).	

	res := db beginTransaction.
	self assert: (res = 0).

	self should: [ db execute: 'insert into x values (NULL, ?, ?)' with: 42 ]
		raise: NBSQLite3Error.

	res := db rollbackTransaction.
	self assert: (res = 0).
	! !

!NBSQLite3ConnectionTest methodsFor: 'tests' stamp: 'PierceNg 4/12/2014 10:07'!
testExecuteWithToFail2
	
	res := db basicExecute: 'create table if not exists x (xk integer primary key, iv integer, tv text);'.
	self assert: (res = 0).
	
	res := db basicExecute: 'delete from x;'.
	self assert: (res = 0).	

	res := db beginTransaction.
	self assert: (res = 0).

	self should: [ db execute: 'insert into x values (NULL, ?, ?)' with: '42' ]
		raise: NBSQLite3Error.

	res := db rollbackTransaction.
	self assert: (res = 0).
	! !

TestCase subclass: #NBSQLite3GlorpAccessorTest
	instanceVariableNames: 'db res rs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Tests-Glorp'!

!NBSQLite3GlorpAccessorTest methodsFor: 'running' stamp: 'PierceNg 9/27/2014 00:37'!
setUp 

	super setUp.
	db := NBSQLite3GlorpAccessor openOn: ':memory:'.
! !

!NBSQLite3GlorpAccessorTest methodsFor: 'running' stamp: 'TorstenBergmann 11/2/2014 20:51'!
tearDown
	
	[ db close ] on: NBSQLite3Misuse do: [ ].
	super tearDown.! !

!NBSQLite3GlorpAccessorTest methodsFor: 'tests' stamp: 'PierceNg 9/27/2014 00:39'!
testUpToEnd
	
	res := db basicExecute: 'create table if not exists x (xk integer primary key);'.
	self assert: (res = 0).
	
	res := db basicExecute: 'delete from x;'.
	self assert: (res = 0).	

	res := db beginTransaction.
	self assert: (res = 0).

	rs := db execute: 'insert into x values (NULL)' with: #().
	self assert: (rs next isNil).
	rs close.

	rs := db execute: 'insert into x values (NULL)' with: #().
	self assert: (rs next isNil).
	rs close.

	res := db commitTransaction.
	self assert: (res = 0).

	res := WriteStream on: (Array new: 10).
	rs := db execute: 'select xk from x'.	
	[ rs atEnd ] whileFalse: [ 
		res nextPut: rs next ].
	rs close.
	self assert: (res contents = #(#(1) #(2)))! !

!NBSQLite3GlorpAccessorTest methodsFor: 'tests - deactivated' stamp: 'PierceNg 10/9/2014 21:39'!
xtestGlorp1
	
	db close.
	db := NBSQLite3GlorpAccessor openOn: 'glorptest.db'.
	
	res := WriteStream on: (Array new: 10).
	rs := db execute: 'select id from stuff where thing = ? or thing = ?' with: #('abc' 'Smalltalk').	
	[ rs atEnd ] whileFalse: [ 
		res nextPut: rs next ].
	rs close.
	self assert: (res contents = #(#(12) #(113141)))! !

NBFFICallback subclass: #NBSQLite3TraceCallback
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Core-Callbacks'!

!NBSQLite3TraceCallback commentStamp: 'PierceNg 7/31/2014 22:52' prior: 0!
I am a Smalltalk-side callback for use with sqlite3_trace().
!

!NBSQLite3TraceCallback class methodsFor: 'accessing' stamp: 'PierceNg 8/2/2014 10:09'!
fnSpec
	"void(*xTrace)(void*, const char*)"
	^ #(void (NBExternalAddress appData, String sqlText))! !

Object subclass: #NBSQLite3BaseConnection
	instanceVariableNames: 'dbFilename dbHandle isOpen'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Core'!

!NBSQLite3BaseConnection commentStamp: 'PierceNg 4/5/2014 19:35' prior: 0!
I represent a connection to an SQLite database. I provide a thin wrapper over the SQLite C API.!

!NBSQLite3BaseConnection class methodsFor: 'instance finalization' stamp: 'PierceNg 4/5/2014 13:05'!
finalizeResourceData: dbHandle
	NBSQLite3FFI current apiClose: dbHandle.
	! !

!NBSQLite3BaseConnection class methodsFor: 'instance creation' stamp: 'TorstenBergmann 11/2/2014 21:07'!
on: aFilename

	^(self new) 
		on: aFilename;
		yourself
! !

!NBSQLite3BaseConnection class methodsFor: 'instance creation' stamp: 'TorstenBergmann 11/2/2014 21:07'!
openOn: aFilename

	^(self new) 
		on: aFilename; 
		open; 
		yourself
! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 4/5/2014 19:37'!
basicExecute: anSQLText 
	^ NBSQLite3FFI current execute: anSQLText on: dbHandle.
	! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 4/9/2014 23:24'!
beginTransaction
	^ self basicExecute: 'begin'	! !

!NBSQLite3BaseConnection methodsFor: 'public API - introspection' stamp: 'PierceNg 7/31/2014 20:42'!
changes
	^ NBSQLite3FFI current changes: dbHandle! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 21:48'!
clearBindings: aStatement
	NBSQLite3FFI current clearBindings: aStatement handle on: dbHandle
	! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 9/20/2014 17:15'!
close

	dbHandle isNil ifFalse: [ NBSQLite3FFI current close: dbHandle ].
	dbHandle := nil.
	isOpen := false! !

!NBSQLite3BaseConnection methodsFor: 'public API - introspection' stamp: 'PierceNg 4/13/2014 01:07'!
columnNamesFor: aTableName
	
	self isOpen
		ifFalse: [ NBSQLite3NotOpen signal ]
		ifTrue: [ 
			| s tableExists |
			s := self prepare: 'select count(*) from sqlite_master where type = "table" and tbl_name = ?'.
			s at: 1 putString: aTableName.
	  		s basicExecute: [ :row | tableExists := ((row integerAt: 0) = 1) ].
	  		s finalize.
			tableExists ifTrue: [ 
				| columns |
				columns := OrderedCollection new.
				s := self prepare: 'pragma table_info(', aTableName, ')'.
				s basicExecute: [ :row | columns add: (row stringAt: 1) ].
				s finalize.
				^ columns ].
			^ nil ]
		! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 4/9/2014 23:24'!
commitTransaction
	^ self basicExecute: 'commit'	! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 1/3/2014 15:54'!
finalize: aStatement
	NBSQLite3FFI current finalize: aStatement handle on: dbHandle
	! !

!NBSQLite3BaseConnection methodsFor: 'initialization' stamp: 'TorstenBergmann 11/2/2014 20:38'!
initialize
	"Initializes the receiver"
	
	super initialize.
	dbHandle := NBExternalObject new.
	isOpen := false.! !

!NBSQLite3BaseConnection methodsFor: 'testing' stamp: 'PierceNg 3/29/2014 19:40'!
isOpen
	^ isOpen! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 3/29/2014 20:19'!
on: aFilename

	dbFilename := aFilename.
	^ self! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 9/20/2014 17:18'!
open

	((NBSQLite3FFI current open: dbFilename via: dbHandle) = 0) 
		ifTrue: [ 
			isOpen := true.
			^ self ]
		ifFalse: [
			| err |
			err := NBSQLite3FFI current apiErrorMessage: dbHandle.
			NBSQLite3FFI current apiClose: dbHandle.
			NBSQLite3Error signal: err ]! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 9/20/2014 17:18'!
prepare: anSQLText 
	| s |
	
	s := NBSQLite3Statement new.
	s connection: self.
	NBSQLite3FFI current prepare: s handle on: dbHandle with: anSQLText.
	^ s
	! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 22:29'!
reset: aStatement
	NBSQLite3FFI current reset: aStatement handle on: dbHandle
	! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 4/9/2014 23:24'!
rollbackTransaction
	^ self basicExecute: 'rollback'	! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 23:11'!
step: aStatement
	NBSQLite3FFI current step: aStatement handle on: dbHandle
	! !

!NBSQLite3BaseConnection methodsFor: 'public API - introspection' stamp: 'PierceNg 4/13/2014 01:07'!
tableNames
	| tables s |
	
	self isOpen
		ifFalse: [ NBSQLite3NotOpen signal ]
		ifTrue: [ 
			tables := OrderedCollection new.
			s := self prepare: 'select tbl_name from sqlite_master where type = "table"'.
	  		s basicExecute: [ :row | tables add: (row stringAt: 0) ].
	  		s finalize.
			^ tables ]! !

!NBSQLite3BaseConnection methodsFor: 'public API - introspection' stamp: 'PierceNg 7/31/2014 20:42'!
totalChanges
	^ NBSQLite3FFI current totalChanges: dbHandle! !

!NBSQLite3BaseConnection methodsFor: 'public API' stamp: 'PierceNg 8/2/2014 09:27'!
traceUsing: callback with: appdata
	NBSQLite3FFI current trace: dbHandle with: callback with: appdata
	! !

NBSQLite3BaseConnection subclass: #NBSQLite3Connection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Core'!

!NBSQLite3Connection commentStamp: 'PierceNg 4/5/2014 19:34' prior: 0!
I represent a connection to an SQLite database.
!

!NBSQLite3Connection methodsFor: 'public API' stamp: 'PierceNg 4/13/2014 11:08'!
execute: anSQLText	
	^ self execute: anSQLText with: #()! !

!NBSQLite3Connection methodsFor: 'public API' stamp: 'TorstenBergmann 11/2/2014 20:53'!
execute: anSQLText with: anObject

	anObject isString
		ifTrue: [ NBSQLite3Error signal: 'Unable to execute SQL on instance of String.' ].
			
	^ NBSQLite3ResultSet new execute: anSQLText with: anObject on: self	
! !

NBSQLite3Connection subclass: #NBSQLite3GlorpAccessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Glorp'!

!NBSQLite3GlorpAccessor commentStamp: 'PierceNg 9/21/2014 11:12' prior: 0!
I am the NBSQLIte3 Glorp database accessor.
!

!NBSQLite3GlorpAccessor methodsFor: 'executing' stamp: 'PierceNg 9/21/2014 15:04'!
basicExecuteSQLString: aString

	^ self basicExecuteSQLString: aString
		returnResult: true
		binding: #()
		doing: [ ]
		
	! !

!NBSQLite3GlorpAccessor methodsFor: 'executing' stamp: 'PierceNg 9/21/2014 14:06'!
basicExecuteSQLString: aString binding: aBindingArray

	^ self basicExecuteSQLString: aString
		returnResult: true
		binding: aBindingArray
		doing: [ ]! !

!NBSQLite3GlorpAccessor methodsFor: 'executing' stamp: 'PierceNg 9/21/2014 14:05'!
basicExecuteSQLString: aString returnResult: returnResult binding: aBindingArray

	^ self basicExecuteSQLString: aString
		returnResult: returnResult
		binding: aBindingArray
		doing: [ ]! !

!NBSQLite3GlorpAccessor methodsFor: 'executing' stamp: 'PierceNg 9/21/2014 15:17'!
basicExecuteSQLString: aString returnResult: returnResult binding: aBindingArray doing: aBlock
	| rs |
	
	rs := self execute: aString with: aBindingArray.
	aBlock value.

	returnResult 
		ifFalse: [ ^ nil ]
		ifTrue: [ ^ GlorpCursoredStream on: rs accessor: self ]
! !

!NBSQLite3GlorpAccessor methodsFor: 'executing' stamp: 'PierceNg 9/21/2014 22:29'!
basicExecuteSQLStringNoResult: aString doing: aBlock

	^self basicExecuteSQLString: aString 
		returnResult: false 
		binding: #() 
		doing: aBlock.
	! !

!NBSQLite3GlorpAccessor methodsFor: 'executing' stamp: 'PierceNg 9/21/2014 15:45'!
completedCommand: aCommand

	"Cribbed from DatabaseAccessor's."
	^ self! !

!NBSQLite3GlorpAccessor methodsFor: 'login' stamp: 'PierceNg 9/21/2014 14:38'!
connect: aLogin

	self on: aLogin host, aLogin databaseName.
	self open
	
! !

!NBSQLite3GlorpAccessor methodsFor: 'accessing' stamp: 'PierceNg 9/21/2014 14:38'!
connection
	^ self! !

!NBSQLite3GlorpAccessor methodsFor: 'login' stamp: 'PierceNg 9/22/2014 00:14'!
disconnect
	self close! !

!NBSQLite3GlorpAccessor methodsFor: 'executing' stamp: 'PierceNg 9/21/2014 20:58'!
execute: anSQLText with: anObject

	(anObject isKindOf: String)
		ifTrue: [ NBSQLite3Error signal: 'Unable to execute SQL on instance of String.' ].
			
	^ NBSQLite3GlorpResultSet new execute: anSQLText with: anObject on: self	
! !

!NBSQLite3GlorpAccessor methodsFor: 'testing' stamp: 'PierceNg 9/21/2014 14:39'!
isConnected
	^ self isOpen! !

!NBSQLite3GlorpAccessor methodsFor: 'testing' stamp: 'PierceNg 9/21/2014 15:17'!
isCursorAtEnd: aCursor
	^ aCursor atEnd! !

!NBSQLite3GlorpAccessor methodsFor: 'login' stamp: 'PierceNg 9/21/2014 11:25'!
logout
	self close! !

!NBSQLite3GlorpAccessor methodsFor: 'executing' stamp: 'PierceNg 9/21/2014 15:45'!
releaseCursor: aCursor in: aPreparedStatement

	"Cribbed from PharoDatabaseAccessor's."
	aCursor isNil ifTrue: [ ^ self ].
	^ [ aCursor close. true ] on: Dialect error do: [ :ex | ex return: false ]! !

!NBSQLite3GlorpAccessor methodsFor: 'executing' stamp: 'PierceNg 9/27/2014 17:47'!
rowCount
	^ self changes! !

Object subclass: #NBSQLite3Constants
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Core'!

!NBSQLite3Constants commentStamp: '<historical>' prior: 0!
I provide SQLIte's constants.!

!NBSQLite3Constants class methodsFor: 'constants' stamp: 'PierceNg 3/30/2014 16:05'!
BLOB
	^ 4! !

!NBSQLite3Constants class methodsFor: 'constants' stamp: 'PierceNg 2/27/2014 22:17'!
BUSY
	^ 5! !

!NBSQLite3Constants class methodsFor: 'constants' stamp: 'PierceNg 3/30/2014 16:05'!
FLOAT
	^ 2! !

!NBSQLite3Constants class methodsFor: 'constants' stamp: 'PierceNg 3/30/2014 16:05'!
INTEGER
	^ 1! !

!NBSQLite3Constants class methodsFor: 'constants' stamp: 'PierceNg 2/27/2014 22:17'!
LOCKED
	^ 6! !

!NBSQLite3Constants class methodsFor: 'constants' stamp: 'PierceNg 4/5/2014 13:21'!
MISUSE
	^ 21! !

!NBSQLite3Constants class methodsFor: 'constants' stamp: 'PierceNg 3/30/2014 16:05'!
NULL
	^ 5! !

!NBSQLite3Constants class methodsFor: 'constants' stamp: 'PierceNg 4/5/2014 12:40'!
OK
	^ 0! !

!NBSQLite3Constants class methodsFor: 'constants' stamp: 'PierceNg 3/30/2014 16:05'!
TEXT
	^ 3! !

Object subclass: #NBSQLite3Example
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Examples'!

!NBSQLite3Example commentStamp: '<historical>' prior: 0!
I implement some examples of using this package.!

!NBSQLite3Example class methodsFor: 'examples' stamp: 'TorstenBergmann 11/2/2014 20:56'!
exampleMultithreading
	"self exampleMultithreading"
		
	| db db2 sem blocking blocked |
	
	db := NBSQLite3BaseConnection on: 'file::memory:?cache=shared'.
	db2 := NBSQLite3BaseConnection on: 'file::memory:?cache=shared'.
	sem := Semaphore new.	

	Transcript open; clear.	
	
	blocking :=	[ :x :sema |
		Transcript show: 'Thread-1: begin exclusive...'; cr; flush.
		[ x open.
			x basicExecute: 'begin exclusive'.
			sema wait.
			Transcript show: 'Thread-1: releasing exclusive lock.'; cr; flush.
			x basicExecute: 'rollback' 
		] ensure: [ x close ] 
	] newProcessWith: (Array with: db with: sem).

	blocking resume.
	Processor yield.
	
	blocked := [ :x |
		Transcript show: 'Thread-2: create table ...'; cr; flush.
		[[ x open. x basicExecute: 'create table if not exists x (xk integer)' ]
		on: NBSQLite3Locked do: [ :ex | Transcript show: ex messageText; cr; flush ]
		] ensure: [ x close ]
	] newProcessWith: (Array with: db2).
	
	blocked resume.
	Processor yield.

	sem signal.
! !

Object subclass: #NBSQLite3FFI
	instanceVariableNames: ''
	classVariableNames: 'DataTypeMap Singleton TypeMap'
	poolDictionaries: ''
	category: 'NBSQLite3-Core'!

!NBSQLite3FFI commentStamp: '<historical>' prior: 0!
I am the FFI interface to SQLite.!

!NBSQLite3FFI class methodsFor: 'factory' stamp: 'PierceNg 1/3/2014 14:46'!
current
	^ Singleton ifNil: [ Singleton := self basicNew ]! !

!NBSQLite3FFI class methodsFor: 'class initialization' stamp: 'PierceNg 3/30/2014 17:09'!
initialize

	self initializeTypeMap.
	self initializeDataTypeMap.
! !

!NBSQLite3FFI class methodsFor: 'class initialization' stamp: 'PierceNg 4/12/2014 00:25'!
initializeDataTypeMap

	DataTypeMap := Dictionary newFromPairs: #(
		1			#integerAt: 
		2			#floatAt:
		3			#stringAt:
		4			#byteArrayAt:
		5			#nilAt:
	)! !

!NBSQLite3FFI class methodsFor: 'class initialization' stamp: 'PierceNg 4/5/2014 18:12'!
initializeTypeMap

	TypeMap := Dictionary newFromPairs: #(
		sqlite3		      NBExternalObject
		sqlite3_stmt	NBExternalObject
		sqlite_int64	NBInt64
	)! !

!NBSQLite3FFI class methodsFor: 'factory' stamp: 'PierceNg 1/3/2014 12:35'!
nbBindingOf: aTypeName
	^ TypeMap at: aTypeName ifAbsent: [ super nbBindingOf: aTypeName ]! !

!NBSQLite3FFI class methodsFor: 'class initialization' stamp: 'PierceNg 4/5/2014 18:13'!
reset
	Singleton := nil! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 2/23/2014 20:10'!
apiBindBlob: aStatement atColumn: aColumn with: aByteArray with: anInteger with: aPointer
	"int sqlite3_bind_blob(sqlite3_stmt*, int, const void *, int, void(*)(void*))"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_bind_blob (sqlite3_stmt aStatement, int aColumn, ByteArray aByteArray, int anInteger, int aPointer)) "SQLITE_TRANSIENT"
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 3/30/2014 17:06'!
apiBindDouble: aStatement atColumn: aColumn with: aFloat
	"int sqlite3_bind_double(sqlite3_stmt*, int, double)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_bind_double (sqlite3_stmt aStatement, int aColumn, double aFloat))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 2/16/2014 00:31'!
apiBindInteger: aStatement atColumn: aColumn with: anInteger
	"int sqlite3_bind_int64(sqlite3_stmt*, int, sqlite_int64)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_bind_int64 (sqlite3_stmt aStatement, int aColumn, sqlite_int64 anInteger))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 4/11/2014 23:39'!
apiBindNull: aStatement atColumn: aColumn
	"int sqlite3_bind_null(sqlite3_stmt*, int)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_bind_null (sqlite3_stmt aStatement, int aColumn))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 4/11/2014 21:40'!
apiBindParameterCount: aStatement
	"int sqlite3_bind_parameter_count(sqlite3_stmt*)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_bind_parameter_count (sqlite3_stmt aStatement))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 4/12/2014 09:13'!
apiBindParameterIndex: aStatement for: aName
	"int sqlite3_bind_parameter_index(sqlite3_stmt*, const char* zName)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_bind_parameter_index (sqlite3_stmt aStatement, String aName))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 21:42'!
apiBindString: aStatement atColumn: aColumn with: aString with: anInteger with: anotherInteger
	"int sqlite3_bind_text(sqlite3_stmt*, int, const char*, int, void(*)(void*))"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_bind_text (sqlite3_stmt aStatement, int aColumn, String aString, int anInteger, int anotherInteger))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 7/31/2014 20:34'!
apiChanges: handle
	"int sqlite3_changes(sqlite3*)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_changes (sqlite3 handle))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 21:35'!
apiClearBindings: aStatement
	"int sqlite3_clear_bindings(sqlite3_stmt*)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_clear_bindings (sqlite3_stmt aStatement))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 21:07'!
apiClose: handle
	"int sqlite3_close_v2(sqlite3*)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_close_v2 (sqlite3 handle))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 2/23/2014 20:12'!
apiColumnBlob: aStatement atColumn: aColumn
	"const void *sqlite3_column_blob(sqlite3_stmt*, int)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(const void *sqlite3_column_blob (sqlite3_stmt aStatement, int aColumn))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 22:42'!
apiColumnBytes: aStatement atColumn: aColumn
	"int sqlite3_column_bytes(sqlite3_stmt*, int)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_column_bytes (sqlite3_stmt aStatement, int aColumn))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 3/30/2014 17:07'!
apiColumnDouble: aStatement atColumn: aColumn
	"double sqlite3_column_double(sqlite3_stmt*, int)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(double sqlite3_column_double (sqlite3_stmt aStatement, int aColumn))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 23:31'!
apiColumnInt: aStatement atColumn: aColumn
	"int sqlite3_column_int(sqlite3_stmt*, int)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_column_int (sqlite3_stmt aStatement, int aColumn))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 3/30/2014 16:08'!
apiColumnName: aStatement atColumn: aColumn
	"const char *sqlite3_column_name(sqlite3_stmt*, int)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(String sqlite3_column_name (sqlite3_stmt aStatement, int aColumn))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 22:41'!
apiColumnText: aStatement atColumn: aColumn
	"const unsigned char *sqlite3_column_text(sqlite3_stmt*, int)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(String sqlite3_column_text (sqlite3_stmt aStatement, int aColumn))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 3/30/2014 16:07'!
apiColumnType: aStatement atColumn: aColumn
	"int sqlite3_column_type(sqlite3_stmt*, int)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_column_type (sqlite3_stmt aStatement, int aColumn))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 3/30/2014 15:47'!
apiDataCount: handle
	"int sqlite3_data_count(sqlite3_stmt*)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_data_count (sqlite3_stmt handle))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 9/20/2014 16:23'!
apiEnableSharedCache: int
	"int sqlite3_enable_shared_cache(int)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_enable_shared_cache (int))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 21:15'!
apiErrorMessage: handle 
	"const char *sqlite3_errmsg(sqlite3*)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(String sqlite3_errmsg (sqlite3 handle))
		module: self library	! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 2/15/2014 20:45'!
apiExec: dbHandle withSQLText: anSQLText with: callback with: callbackData with: errorMessageBuffer
	"int sqlite3_exec(sqlite3*, const char*, int (*callback), void*, char **)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_exec (sqlite3 dbHandle, String anSQLText, void *callback, void *callbackData, char **errorMessageBuffer))
		module: self library
	! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 21:06'!
apiFinalize: handle
	"int sqlite3_finalize(sqlite3_stmt*)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_finalize (sqlite3_stmt handle))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 21:14'!
apiOpen: filename via: handle
	"int sqlite3_open(const char*, sqlite3**)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_open (String filename, sqlite3* handle))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 21:25'!
apiPrepare: dbHandle withSQLText: anSQLText with: textSize with: sHandle with: tail
	"int sqlite3_prepare_v2(sqlite3*, const char*, int, sqlite3_stmt**, const char **)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_prepare_v2 (sqlite3 dbHandle, String anSQLText, int textSize, sqlite3_stmt* sHandle, const char** tail))
		module: self library
	! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 23:13'!
apiReset: handle
	"int sqlite3_reset(sqlite3_stmt*)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_reset (sqlite3_stmt handle))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 1/4/2014 23:12'!
apiStep: handle
	"int sqlite3_step(sqlite3_stmt*)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_step (sqlite3_stmt handle))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 2/26/2014 21:14'!
apiThreadsafe
	"int sqlite3_threadsafe(void)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_threadsafe ())
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 7/31/2014 20:38'!
apiTotalChanges: handle
	"int sqlite3_total_changes(sqlite3*)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCall: #(int sqlite3_total_changes (sqlite3 handle))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 8/2/2014 09:27'!
apiTrace: handle with: callback with: appdata
	"void *sqlite3_trace(sqlite3* void(*)(void*, const char*), void*)"
	<primitive: #primitiveNativeCall module: #NativeBoostPlugin error: errorCode>
	^ self nbCallout 
		options: #( optMayGC );
		function: #(void* sqlite3_trace (sqlite3 handle, NBSQLite3TraceCallback callback, NBExternalAddress appdata))
		module: self library
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 4/11/2014 21:41'!
bindParameterCount: aStatementHandle
	^ self apiBindParameterCount: aStatementHandle
	
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 4/12/2014 09:14'!
bindParameterIndex: aStatementHandle for: aName
	^ self apiBindParameterIndex: aStatementHandle for: aName
	
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 2/23/2014 20:14'!
blobFrom: aStatement at: aColumn
	| address size |
	
	address := self apiColumnBlob: aStatement atColumn: aColumn.
	size := self apiColumnBytes: aStatement atColumn: aColumn.
	^ (((NBExternalArray ofType: NBCharacterType) onAddress: address size: size) postCopy; yourself) address! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 7/31/2014 20:35'!
changes: aHandle	
	^ self apiChanges: aHandle
	
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 4/5/2014 13:22'!
checkForOk: aValue on: aHandle

	(aValue = 0)
		ifFalse: [ 
			(aValue = NBSQLite3Constants BUSY)
				ifTrue: [ self signal: NBSQLite3Busy with: aValue on: aHandle ].
			(aValue = NBSQLite3Constants LOCKED)
				ifTrue: [ self signal: NBSQLite3Locked with: aValue on: aHandle ].
			(aValue = NBSQLite3Constants MISUSE)
				ifTrue: [ self signal: NBSQLite3Misuse with: aValue on: aHandle ].				
			self signal: NBSQLite3Error with: aValue on: aHandle ].	
	^ 0! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 4/5/2014 13:38'!
clearBindings: sHandle on: dbHandle	
	^ self checkForOk: (self apiClearBindings: sHandle) on: dbHandle
	
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 4/5/2014 13:38'!
close: aHandle	
	^ self checkForOk: (self apiClose: aHandle) on: aHandle
	
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 3/30/2014 16:10'!
columnNameFrom: aStatement at: aColumn
	^ self apiColumnType: aStatement atColumn: aColumn! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 3/30/2014 16:10'!
columnTypeFrom: aStatement at: aColumn
	^ self apiColumnType: aStatement atColumn: aColumn! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 3/30/2014 15:55'!
dataValuesAvailable: aStatementHandle
	^ self apiDataCount: aStatementHandle
	
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 2/24/2014 20:44'!
execute: anSQLText on: aDBHandle
	| null |
	
	null := NBExternalAddress null.
	^ self checkForOk: (self apiExec: aDBHandle 
							withSQLText: anSQLText
							with: null
							with: null
							with: null)
		 on: aDBHandle
	
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 4/5/2014 13:38'!
finalize: aStatementHandle on: aDBHandle	
	^ self checkForOk: (self apiFinalize: aStatementHandle) on: aDBHandle
	
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 3/30/2014 16:54'!
floatFrom: aStatement at: aColumn
	^ self apiColumnDouble: aStatement atColumn: aColumn! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 22:53'!
integerFrom: aStatement at: aColumn
	^ self apiColumnInt: aStatement atColumn: aColumn! !

!NBSQLite3FFI methodsFor: 'accessing' stamp: 'PierceNg 1/3/2014 14:01'!
library
	| platform |
	
	platform := NativeBoost forCurrentPlatform class.
	platform = NativeBoostMac32 ifTrue: [ ^ #sqlite3 ].
	^ 'sqlite3'! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 3/30/2014 16:12'!
nameFor: aStatement at: aColumn
	^ self apiColumnName: aStatement atColumn: aColumn! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 4/5/2014 13:37'!
open: aFilename via: aHandle	
	^ self checkForOk: (self apiOpen: aFilename via: aHandle) on: aHandle
	
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 4/5/2014 13:39'!
prepare: aStatementHandle on: aDBHandle with: anSQLText

	^ self checkForOk: (self apiPrepare: aDBHandle 
							withSQLText: anSQLText
							with: (anSQLText size)
							with: aStatementHandle
							with: (NBExternalAddress null))
		 on: aDBHandle
	
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 4/5/2014 13:39'!
reset: aStatementHandle on: aDBHandle	
	^ self checkForOk: (self apiReset: aStatementHandle) on: aDBHandle
	
! !

!NBSQLite3FFI methodsFor: 'private' stamp: 'PierceNg 2/27/2014 22:21'!
signal: exceptionClass with: value on: handle
	exceptionClass new resultCode: value; signal: (self apiErrorMessage: handle)! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 23:22'!
step: aStatementHandle	
	^ self apiStep: aStatementHandle
	
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 22:52'!
stringFrom: aStatement at: aColumn
	^ self apiColumnText: aStatement atColumn: aColumn! !

!NBSQLite3FFI methodsFor: 'accessing' stamp: 'PierceNg 2/26/2014 21:15'!
threadsafe
	^ self apiThreadsafe! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 7/31/2014 20:39'!
totalChanges: aHandle	
	^ self apiTotalChanges: aHandle
	
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 8/2/2014 09:27'!
trace: handle with: callback with: appdata
	^ self apiTrace: handle with: callback with: appdata
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 4/12/2014 00:07'!
typeFor: aStatement at: aColumn
	^ DataTypeMap at: (self apiColumnType: aStatement atColumn: aColumn)
! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 2/23/2014 16:15'!
with: aStatement at: aColumn putBlob: aByteArray
	^ self apiBindBlob: aStatement atColumn: aColumn with: aByteArray with: aByteArray size with: -1 ! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 3/30/2014 16:57'!
with: aStatement at: aColumn putFloat: aFloat
	^ self apiBindDouble: aStatement atColumn: aColumn with: aFloat! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 21:39'!
with: aStatement at: aColumn putInteger: anInteger
	^ self apiBindInteger: aStatement atColumn: aColumn with: anInteger   ! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 1/4/2014 21:41'!
with: aStatement at: aColumn putString: aString
	^ self apiBindString: aStatement atColumn: aColumn with: aString with: -1 with: -1 ! !

!NBSQLite3FFI methodsFor: 'public API' stamp: 'PierceNg 4/11/2014 23:41'!
with: aStatement putNullAt: aColumn 
	^ self apiBindNull: aStatement atColumn: aColumn! !

Object subclass: #NBSQLite3ResultSet
	instanceVariableNames: 'db statement moreRows'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Core'!

!NBSQLite3ResultSet commentStamp: 'PierceNg 4/13/2014 12:33' prior: 0!
I represent a set of results returned by the database. I produce NBSQLite3Row instances lazily.
!

NBSQLite3ResultSet subclass: #NBSQLite3GlorpResultSet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Glorp'!

!NBSQLite3GlorpResultSet commentStamp: 'PierceNg 9/21/2014 20:56' prior: 0!
I represent a set of results returned by the database. I produce my rows lazily. As required by Glorp, each row is an array, a la a tuple.!

!NBSQLite3GlorpResultSet methodsFor: 'API' stamp: 'PierceNg 9/21/2014 21:00'!
atEnd
	^ moreRows not! !

!NBSQLite3GlorpResultSet methodsFor: 'API' stamp: 'PierceNg 9/27/2014 10:15'!
next
	| nc rr |

	statement ifNil: [ NBSQLite3Misuse signal: 'This result set does not contain a prepared statement.' ].	
	
	moreRows ifTrue: [	
		nc := statement dataValuesAvailable.
		(nc = 0)
			ifTrue: [ ^ nil ] 
			ifFalse: [ 
				rr := OrderedCollection new.
				0 to: nc - 1 do: [ :c | rr add: (statement valueOfColumn: c) ].
				"statement clearBindings."
				moreRows := (statement step) = 100. " == SQLITE_ROW "
				^ rr asArray ]].
		
	^ nil

! !

!NBSQLite3ResultSet methodsFor: 'API' stamp: 'PierceNg 4/17/2014 00:32'!
close

	statement ifNotNil: [ statement finalize ].
	statement := nil! !

!NBSQLite3ResultSet methodsFor: 'private' stamp: 'PierceNg 9/23/2014 23:02'!
execute: anSQLText with: anObject on: aDb
	| obj |

	db := aDb.

	obj := anObject isNil ifTrue: [ #() ] ifFalse: [ anObject ].

	(obj isCollection)
		ifTrue: [ self execute: anSQLText withCollection: obj ]
		ifFalse: [
			(anObject respondsTo: #keysAndValuesDo:)
				ifTrue: [ self execute: anSQLText withKeysAndValues: obj ]
				ifFalse: [ NBSQLite3Error signal: 'Unable to execute SQL on instance of ', obj class asString ]]
	

! !

!NBSQLite3ResultSet methodsFor: 'private' stamp: 'PierceNg 10/2/2014 06:51'!
execute: anSQLText withCollection: aCollection
	| i |

	self prepareStatement: anSQLText.
	i := 1.
	aCollection do: [ :v | i := self execute: statement withIndex: i withValue: v ].
	moreRows := (statement step) = 100. " == SQLITE_ROW "

! !

!NBSQLite3ResultSet methodsFor: 'private' stamp: 'PierceNg 4/13/2014 11:19'!
execute: aStatement withIndex: anInteger withValue: anObject
		
	aStatement perform: (aStatement dataTypeForObject: anObject) with: anInteger with: anObject.
	^ anInteger + 1
		
! !

!NBSQLite3ResultSet methodsFor: 'private' stamp: 'PierceNg 4/13/2014 11:20'!
execute: aStatement withKey: keyObject withValue: valueObject
	| col |

	col := aStatement bindParameterIndex: keyObject.
	(col = 0) ifFalse: [
		aStatement perform: (aStatement dataTypeForObject: valueObject) with: col with: valueObject ]

		
! !

!NBSQLite3ResultSet methodsFor: 'private' stamp: 'PierceNg 10/2/2014 06:51'!
execute: anSQLText withKeysAndValues: anObject
	
	self prepareStatement: anSQLText.
	anObject keysAndValuesDo: [ :k :v | 
		self execute: statement withKey: k withValue: v ].
	moreRows := (statement step) = 100. " == SQLITE_ROW "
! !

!NBSQLite3ResultSet methodsFor: 'initialization' stamp: 'TorstenBergmann 11/2/2014 20:39'!
initialize
	"Initializes the receiver"
	
	super initialize.
	moreRows := true
! !

!NBSQLite3ResultSet methodsFor: 'API' stamp: 'PierceNg 9/27/2014 10:17'!
next
	| nc rr |

	statement ifNil: [ NBSQLite3Misuse signal: 'This result set does not contain a prepared statement.' ].	
	
	moreRows ifTrue: [	
		nc := statement dataValuesAvailable.
		(nc = 0)
			ifTrue: [ ^ nil ] 
			ifFalse: [ 
				rr := NBSQLite3Row new.
				0 to: nc - 1 do: [ :c | rr at: (statement nameOfColumn: c) put: (statement valueOfColumn: c) ].
				"statement clearBindings." "XXX With this, the Glorp result set fails its test..."
				moreRows := (statement step) = 100. " == SQLITE_ROW "
				^ rr ]].
		
	^ nil
! !

!NBSQLite3ResultSet methodsFor: 'private' stamp: 'PierceNg 10/2/2014 06:51'!
prepareStatement: anSQLText

	statement isNil
		ifTrue: [ statement := db prepare: anSQLText ]
		ifFalse: [ statement clearBindings; reset ].
! !

!NBSQLite3ResultSet methodsFor: 'accessing' stamp: 'PierceNg 10/2/2014 06:48'!
statement
	^ statement! !

!NBSQLite3ResultSet methodsFor: 'accessing' stamp: 'PierceNg 10/2/2014 06:48'!
statement: anObject
	statement := anObject! !

Object subclass: #NBSQLite3Row
	instanceVariableNames: 'data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Core'!

!NBSQLite3Row commentStamp: 'PierceNg 4/5/2014 19:32' prior: 0!
I represent a row of results returned by the database.
!

!NBSQLite3Row methodsFor: 'accessing' stamp: 'PierceNg 4/13/2014 00:27'!
at: aKey 
	^ self at: aKey ifAbsent: [  ]! !

!NBSQLite3Row methodsFor: 'accessing' stamp: 'PierceNg 4/13/2014 00:26'!
at: aKey ifAbsent: aBlock
	^ data at: aKey ifAbsent: aBlock! !

!NBSQLite3Row methodsFor: 'accessing' stamp: 'PierceNg 4/13/2014 00:13'!
at: aKey put: anObject
	data at: aKey put: anObject! !

!NBSQLite3Row methodsFor: 'reflective operations' stamp: 'TorstenBergmann 11/2/2014 21:00'!
doesNotUnderstand: aMessage

	^self 
		at: aMessage selector asString 
		ifAbsent: [ super doesNotUnderstand: aMessage ]! !

!NBSQLite3Row methodsFor: 'initialization' stamp: 'TorstenBergmann 11/2/2014 20:39'!
initialize
	"Initializes the receiver"
	
	super initialize.
	data := Dictionary new! !

Object subclass: #NBSQLite3Statement
	instanceVariableNames: 'connection handle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Core'!

!NBSQLite3Statement commentStamp: '<historical>' prior: 0!
I represent an SQLite prepared statement.!

!NBSQLite3Statement class methodsFor: 'instance finalization' stamp: 'PierceNg 4/13/2014 17:00'!
finalizeResourceData: aHandle
	NBSQLite3FFI current apiFinalize: aHandle.! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 3/30/2014 16:49'!
at: aColumn putByteArray: anObject
	^ NBSQLite3FFI current with: handle at: aColumn putBlob: anObject! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 4/6/2014 08:36'!
at: aColumn putFloat: aFloat
	^ NBSQLite3FFI current with: handle at: aColumn putFloat: aFloat! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 2/16/2014 00:25'!
at: aColumn putInteger: anInteger
	^ NBSQLite3FFI current with: handle at: aColumn putInteger: anInteger! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 4/12/2014 10:20'!
at: aColumn putNil: anObject
	"Goofy interface note - anObject is ignored here. When used by NBSQLite3Connection, anObject will always be nil."
	^ NBSQLite3FFI current with: handle putNullAt: aColumn! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 2/23/2014 16:21'!
at: aColumn putObject: anObject
	^ NBSQLite3FFI current with: handle at: aColumn putBlob: (FLSerializer serializeToByteArray: anObject)! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 2/16/2014 00:26'!
at: aColumn putString: aString
	^ NBSQLite3FFI current with: handle at: aColumn putString: aString! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 4/12/2014 23:35'!
basicExecute: aBlock
	| ret |
	
	ret := self step. 
	(ret = 101) " == SQLITE_DONE, meaning step should not be sent again."
		ifTrue: [ aBlock value: self ]
		ifFalse: [
			[ ret = 100 ] whileTrue: [ " == SQLITE_ROW, meaning another row is available."
				aBlock value: self.
				ret := self step ]]! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 4/11/2014 21:42'!
bindParameterCount
	^ NBSQLite3FFI current bindParameterCount: handle! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 4/12/2014 21:04'!
bindParameterIndex: aName
	^ NBSQLite3FFI current bindParameterIndex: handle for: aName! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 3/30/2014 16:49'!
byteArrayAt: aColumn
	^ NBSQLite3FFI current blobFrom: handle at: aColumn! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 1/4/2014 21:47'!
clearBindings
	connection clearBindings: self! !

!NBSQLite3Statement methodsFor: 'private' stamp: 'PierceNg 1/3/2014 15:50'!
connection: anObject
	connection := anObject! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 9/25/2014 06:42'!
dataTypeForObject: anObject

	anObject ifNil: [ ^#at:putNil: ].

	(anObject isKindOf: Integer)
		ifTrue: [ ^#at:putInteger: ].
		
	(anObject isKindOf: Float)
		ifTrue: [ ^#at:putFloat: ].
		
	(anObject isKindOf: String)
		ifTrue: [ ^#at:putString: ].
		
	(anObject isKindOf: ByteArray)
		ifTrue: [ ^#at:putByteArray: ].
		
	^ #at:putObject: ! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 3/30/2014 15:55'!
dataValuesAvailable
	^ NBSQLite3FFI current dataValuesAvailable: handle! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 9/20/2014 17:13'!
finalize 

	connection finalize: self.
	handle := nil	! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 3/30/2014 17:05'!
floatAt: aColumn
	^ NBSQLite3FFI current floatFrom: handle at: aColumn! !

!NBSQLite3Statement methodsFor: 'private' stamp: 'PierceNg 1/3/2014 16:31'!
handle
	^ handle! !

!NBSQLite3Statement methodsFor: 'initialization' stamp: 'TorstenBergmann 11/2/2014 20:40'!
initialize

	super initialize.
	handle := NBExternalObject new.! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 1/4/2014 23:00'!
integerAt: aColumn
	^ NBSQLite3FFI current integerFrom: handle at: aColumn! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 3/30/2014 16:16'!
nameOfColumn: aColumn
	^ NBSQLite3FFI current nameFor: handle at: aColumn! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 3/30/2014 16:50'!
nilAt: aColumn
	^ nil! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 2/23/2014 16:20'!
objectAt: aColumn
	^ FLMaterializer materializeFromByteArray: (NBSQLite3FFI current blobFrom: handle at: aColumn)! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 4/11/2014 23:52'!
putNilAt: aColumn
	^ NBSQLite3FFI current with: handle putNullAt: aColumn! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 1/4/2014 22:30'!
reset
	connection reset: self! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 1/4/2014 23:28'!
step
	^ NBSQLite3FFI current step: handle! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 1/4/2014 23:00'!
stringAt: aColumn
	^ NBSQLite3FFI current stringFrom: handle at: aColumn! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 3/30/2014 16:26'!
typeOfColumn: aColumn
	^ NBSQLite3FFI current typeFor: handle at: aColumn! !

!NBSQLite3Statement methodsFor: 'public' stamp: 'PierceNg 3/30/2014 17:18'!
valueOfColumn: aColumn
	^ self perform: (self typeOfColumn: aColumn) with: aColumn
! !

Error subclass: #NBSQLite3Error
	instanceVariableNames: 'resultCode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Core-Exceptions'!

!NBSQLite3Error commentStamp: '<historical>' prior: 0!
I represent an SQLite non-ok condition.!

NBSQLite3Error subclass: #NBSQLite3Busy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Core-Exceptions'!

!NBSQLite3Busy commentStamp: '<historical>' prior: 0!
I represent the SQLITE_BUSY condition.!

!NBSQLite3Error methodsFor: 'accessing' stamp: 'PierceNg 2/27/2014 22:11'!
resultCode
	^ resultCode! !

!NBSQLite3Error methodsFor: 'accessing' stamp: 'PierceNg 2/27/2014 22:11'!
resultCode: anObject
	resultCode := anObject! !

NBSQLite3Error subclass: #NBSQLite3Locked
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Core-Exceptions'!

!NBSQLite3Locked commentStamp: '<historical>' prior: 0!
I represent the SQLITE_LOCKED condition.!

NBSQLite3Error subclass: #NBSQLite3Misuse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Core-Exceptions'!

!NBSQLite3Misuse commentStamp: 'PierceNg 4/6/2014 00:01' prior: 0!
I represent the SQLITE_MISUSE condition.
!

NBSQLite3Error subclass: #NBSQLite3NotOpen
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NBSQLite3-Core-Exceptions'!

!NBSQLite3NotOpen commentStamp: 'PierceNg 3/29/2014 20:17' prior: 0!
I am signalled when operations are invoked on a database that is not open.
!
NBSQLite3FFI initialize!
